@@ -16,8 +16,8 @@ class DegenerateSpaceInputFormat(enum.Enum):
     Groups = "groups"
     QuantaSpecRules = "nT"
     Polyads = "polyads"
-    StrongCouplings = "couplings"
-    EnergyCutoff = "energy_cutoff"
+    StrongCouplings = "wfc_threshold"
+    EnergyCutoff = "energy_window"
     Callable = "callable"
     MartinTest = "martin_threshold"
 
@@ -27,9 +27,45 @@ class DegeneracySpec(metaclass=abc.ABCMeta):
     in a way that can be cleanly canonicalized
     """
 
-    format = None
     application_order = 'pre'
     group_filter = None
+    def __init__(self, application_order=None, group_filter=None, energy_cutoff=2.25e-3, test_modes=None, maximize_filtered_groups=True):
+        if application_order is None:
+            application_order = self.application_order
+        self.application_order = application_order
+        self.energy_cutoff = energy_cutoff
+        if group_filter is None:
+            group_filter = self.group_filter
+        self.group_filter = group_filter
+        self.test_modes = test_modes
+        self.maximize_filtered_groups = maximize_filtered_groups
+
+    repr_opts = ['energy_cutoff']
+    def __repr__(self):
+        return "{}({})".format(type(self).__name__, ", ".join('{}={}'.format(k, getattr(self, k)) for k in self.repr_opts))
+
+    def get_degenerate_group_filter(self, solver, corrs=None, threshold=None):
+        group_filter = self.group_filter
+        if isinstance(group_filter, str) and group_filter == 'default':
+            group_filter = None
+        if group_filter is None:
+            zero_order_energy_cutoff = self.energy_cutoff
+            test_modes = self.test_modes
+            if test_modes is None:
+                test_modes = solver.high_frequency_modes
+            group_filter = dict(
+                corrections=corrs,
+                energy_cutoff=zero_order_energy_cutoff,
+                energies=solver.zero_order_energies,
+                threshold=threshold,
+                target_modes=test_modes,
+                maximize_groups=self.maximize_filtered_groups
+            )
+        elif isinstance(group_filter, str) and group_filter == 'unfiltered':
+            group_filter = None
+        return group_filter
+
+    format = None
     @classmethod
     def get_format_mapping(cls):
         return {
@@ -43,9 +79,17 @@ class DegeneracySpec(metaclass=abc.ABCMeta):
         }
 
     @classmethod
+    def get_default_spec(cls):
+        return StronglyCoupledDegeneracySpec()
+    @classmethod
     def from_spec(cls, spec, format=None, **kwargs):
         if spec is None:
             return None
+        elif (
+                isinstance(spec, str) and spec == 'auto'
+                or spec is True
+        ):
+            return cls.get_default_spec()
 
         if format is None:
             (spec, opts), format = cls.infer_format(spec)
@@ -105,7 +149,7 @@ class DegeneracySpec(metaclass=abc.ABCMeta):
                 return (None, None), None
 
     @abc.abstractmethod
-    def get_groups(self, input_states, solver=None):
+    def get_groups(self, input_states, solver=None, **kwargs):
         """
         :param solver:
         :type solver:
@@ -122,13 +166,17 @@ class DegeneracySpec(metaclass=abc.ABCMeta):
     def canonicalize(cls, spec):
         raise NotImplementedError("abstract interface")
 
+    def get_group_filter(self, **kwargs):
+        return DegenerateMultiStateSpace.construct_filer(**kwargs)
+
 class EnergyCutoffDegeneracySpec(DegeneracySpec):
     """
 
     """
 
     format = DegenerateSpaceInputFormat.EnergyCutoff
-    def __init__(self, cutoff):
+    def __init__(self, cutoff, **opts):
+        super().__init__(**opts)
         self.cutoff = cutoff
 
     @classmethod
@@ -167,15 +215,21 @@ class EnergyCutoffDegeneracySpec(DegeneracySpec):
         return degenerate_groups
 
 class MartinTestDegeneracySpec(DegeneracySpec):
-    application_order = 'post'
+    application_order = 'mid'
     group_filter = 'default'
-    def __init__(self, threshold=4.6e-6, energy_cutoff=4.5e-3, convert=True, frequencies=None):
-        if convert and threshold > 1e-2:
+    def __init__(self, threshold=4.6e-6, test_energy_window=4.6e-3, convert=True, frequencies=None, **opts):
+        super().__init__(**opts)
+        if convert and threshold > 1e-2: # help coerce it into Hartrees
             threshold = threshold / 219465 # only need a rough conversion...
         self.threshold = threshold
+        self.window = test_energy_window
         self.frequencies = frequencies
-        self.energy_cutoff = energy_cutoff
         self._test_groups = None
+        self._states = None
+        self._basis = None
+        self._matrix = None
+
+    repr_opts = ['energy_cutoff', 'threshold']
     def prep_states(self, states:BasisStateSpace):
         state_list = states.excitations
         zo_engs = np.dot(state_list, self.frequencies)
@@ -185,7 +239,7 @@ class MartinTestDegeneracySpec(DegeneracySpec):
         for i, (s, e0, exc) in enumerate(zip(state_list, zo_engs, extended_states)):
             e1 = np.dot(exc.excitations, self.frequencies)
             eng_diffs = e1 - e0
-            pos = np.where(np.abs(eng_diffs < self.energy_cutoff))
+            pos = np.where(np.abs(eng_diffs < self.window))
             if len(pos) > 0 and len(pos[0]) > 0:
                 key_inds.append(i)
                 exc_groups.append(exc.take_subspace(pos[0]))
@@ -208,21 +262,32 @@ class MartinTestDegeneracySpec(DegeneracySpec):
         if solver is None:
             raise ValueError("need a solver with fully evaluated perturbations first...")
         total_basis = solver.flat_total_space
-        engs = solver.zero_order_energies
+        # engs = solver.zero_order_energies
         h1 = solver.representations[1]
         groups = self._test_groups #type:SelectionRuleStateSpace
         keys = groups.representative_space
         key_inds = total_basis.find(keys)
         spaces = []
+        martin_inds = [[], []]
+        martin_vals = []
         for i, (k, subspace) in enumerate(zip(key_inds, groups.flat)):
+            e0 = np.dot(keys.excitations[i], self.frequencies)
+            e1 = np.dot(subspace.excitations, self.frequencies)
             p = total_basis.find(subspace)
-            e_diffs = np.abs(engs[p] - engs[k])
+            e_diffs = np.abs(e1 - e0)
             repr_elems = h1[k, p].asarray()
             # diffs = subspace.excitations - keys.excitations[np.newaxis, k]
             # diff_sums = np.sum(np.abs(diffs), axis=1)
             # raise Exception(diffs, keys.excitations[np.newaxis, k], subspace.excitations)
             # weights = 1
             test_val = (repr_elems**4)/(e_diffs**3)
+            # if (keys.excitations[i] == np.array([0, 0, 0, 1, 0, 0, 0, 0, 0])).all():
+            #     raise Exception(test_val*219465, repr_elems, e_diffs, subspace.excitations)
+            # if (keys.excitations[i] == np.array([0, 0, 0, 0, 0, 0, 1, 0, 0])).all():
+            #     raise Exception(test_val*219465, repr_elems, e_diffs, subspace.excitations)
+            martin_inds[0].extend([i]*len(subspace))
+            martin_inds[1].extend(p)
+            martin_vals.extend(test_val)
             pos = np.where(test_val > self.threshold)
             if len(pos) > 0 and len(pos[0]) > 0:
                 # print(pos[0], subspace.excitations)
@@ -231,11 +296,23 @@ class MartinTestDegeneracySpec(DegeneracySpec):
                         subspace.excitations[pos[0],]#.take_subspace(pos[0]).excitations
                     ], axis=0)
                 # print(keys.excitations[i:i+1], new)
+                new_sums = np.sum(new, axis=1)
+                new = new[new_sums > 0]
                 spaces.append(new)
+
+        self._states = keys
+        self._basis = solver.flat_total_space
+        self._matrix = nput.SparseArray.from_data(
+            (
+                np.array(martin_vals),
+                tuple(np.array(x) for x in martin_inds)
+            ),
+            shape=(len(self._states), len(self._basis))
+        )
         # raise Exception("...")
         return spaces
     # @classmethod
-    def get_groups(self, input_states:BasisStateSpace, solver=None):
+    def get_groups(self, input_states:BasisStateSpace, solver=None, **kwargs):
         groups = self.get_coupled_spaces(input_states, solver=solver)
         indexer = SymmetricGroupGenerator(input_states.ndim)
         groups = PermutationRelationGraph.merge_groups([(indexer.to_indices(g), g) for g in groups])
@@ -245,13 +322,100 @@ class MartinTestDegeneracySpec(DegeneracySpec):
     def canonicalize(cls, spec):
         return isinstance(spec.threshold, float)
 
+    def get_group_filter(self, threshold=None, states=None, basis=None, corrections=None, **kwargs):
+        return DegenerateMultiStateSpace.construct_filer(
+            states=self._states,
+            basis=self._basis,
+            corrections=self._matrix,
+            threshold=self.threshold,
+            threshold_step_size=1/219465,
+            **kwargs
+        )
+
+class StronglyCoupledDegeneracySpec(DegeneracySpec):
+    """
+
+    """
+    application_order = 'post'
+    format = DegenerateSpaceInputFormat.StrongCouplings
+    default_threshold=.3
+    def __init__(self, wfc_threshold=None, state_filter=None, extend_spaces=True, **opts):
+        super().__init__(**opts)
+        if wfc_threshold is None or isinstance(wfc_threshold, str) and wfc_threshold == 'auto':
+            wfc_threshold = self.default_threshold
+        self.wfc_threshold = wfc_threshold
+        self.state_filter = state_filter
+        self.extend_spaces=extend_spaces
+
+    repr_opts = ['energy_cutoff', 'wfc_threshold']
+    def prep_states(self, input_states):
+        return input_states
+
+    def identify_strong_couplings(self, solver, corrs):
+        degenerate_correction_threshold = self.wfc_threshold
+        test_modes = self.test_modes
+        if test_modes is None:
+            test_modes = solver.high_frequency_modes
+
+            # raise Exception(test_modes, test_freqs * 219465, fundamentals.excitations, where)
+
+        state_filter = self.state_filter
+        zero_order_energy_cutoff = self.energy_cutoff
+        if state_filter is None:
+            state_filter = lambda state, couplings: corrs.default_state_filter(state, couplings,
+                                                                               energy_cutoff=zero_order_energy_cutoff,
+                                                                               energies=solver.zero_order_energies,
+                                                                               basis=solver.flat_total_space,
+                                                                               target_modes=test_modes
+                                                                               )
+        sc = corrs.find_strong_couplings(threshold=degenerate_correction_threshold, state_filter=state_filter)
+        return sc
+
+    def get_groups(self, input_states, couplings=None, solver=None, **kwargs):
+        """
+        :param input_states:
+        :type input_states:
+        :param solver:
+        :type solver:
+        :return:
+        :rtype:
+        """
+        if couplings is None:
+            raise ValueError("need couplings")
+        return self.get_strong_coupling_space(input_states, couplings)
+
+    @classmethod
+    def canonicalize(cls, spec):
+        if isinstance(spec, (int, float, np.integer, np.floating)):
+            return spec
+        else:
+            try:
+                return {k:GroupsDegeneracySpec._validate_grp(v) for k,v in spec.items()}
+            except (np.VisibleDeprecationWarning, AttributeError, StopIteration):
+                return None
+
+    @classmethod
+    def get_strong_coupling_space(cls, states: BasisStateSpace, couplings: dict):
+        indexer = SymmetricGroupGenerator(states.ndim)
+        groups = [None] * len(states.indices)
+        for n, i in enumerate(states.indices):
+            if i in couplings:
+                groups[n] = [i] + list(couplings[i].indices)
+            else:
+                groups[n] = [i]
+
+        groups = PermutationRelationGraph.merge_groups([(g, indexer.from_indices(g)) for g in groups])
+
+        return [g[1] for g in groups]
+
 class GroupsDegeneracySpec(DegeneracySpec):
     """
 
     """
 
     format = DegenerateSpaceInputFormat.Groups
-    def __init__(self, groups):
+    def __init__(self, groups, **opts):
+        super().__init__(**opts)
         self.groups = groups
 
     @staticmethod
@@ -269,7 +433,7 @@ class GroupsDegeneracySpec(DegeneracySpec):
         except (np.VisibleDeprecationWarning, AttributeError, StopIteration):
             return None
 
-    def get_groups(self, input_states, solver=None):
+    def get_groups(self, input_states, solver=None, **kwargs):
         """
         :param solver:
         :type solver:
@@ -289,8 +453,9 @@ class PolyadDegeneracySpec(DegeneracySpec):
     format = DegenerateSpaceInputFormat.Polyads
     def __init__(self, polyads,
                  max_quanta=None, max_iterations=2,
-                 require_converged=False, extra_groups=None
-                 ):
+                 require_converged=False, extra_groups=None,
+                 **opts):
+        super().__init__(**opts)
         self.polyads = polyads
         self.max_quanta = max_quanta
         self.max_iterations = max_iterations
@@ -317,7 +482,7 @@ class PolyadDegeneracySpec(DegeneracySpec):
         except (np.VisibleDeprecationWarning, AttributeError, StopIteration):
             return None
 
-    def get_groups(self, input_states, solver=None):
+    def get_groups(self, input_states, solver=None, **kwargs):
         """
         :param solver:
         :type solver:
@@ -377,19 +542,18 @@ class PolyadDegeneracySpec(DegeneracySpec):
         except TypeError:
             return False
 
-
 class TotalQuantaDegeneracySpec(PolyadDegeneracySpec):
     """
 
     """
 
     format = DegenerateSpaceInputFormat.QuantaSpecRules
-    def __init__(self, n_T_vectors, max_quanta=3):
+    def __init__(self, n_T_vectors, max_quanta=3, **opts):
         self.max_quanta = max_quanta
         self.nt_vecs = [n_T_vectors] if isinstance(n_T_vectors[0], int) else n_T_vectors
         rules = sum((self.make_nt_polyad(v, max_quanta=max_quanta) for v in self.nt_vecs), [])
         # raise Exception(rules)
-        super().__init__(rules)
+        super().__init__(rules, **opts)
 
     @classmethod
     def reduce_rule(cls, a, b):
@@ -435,57 +599,17 @@ class TotalQuantaDegeneracySpec(PolyadDegeneracySpec):
                         rules.add((tuple(a), tuple(b)))
         return list(rules)
 
-class StronglyCoupledDegeneracySpec(DegeneracySpec):
-    """
-
-    """
-
-    format = DegenerateSpaceInputFormat.StrongCouplings
-    def __init__(self, couplings):
-        self.couplings = couplings
-
-    def get_groups(self, input_states, solver=None):
-        """
-        :param input_states:
-        :type input_states:
-        :param solver:
-        :type solver:
-        :return:
-        :rtype:
-        """
-        return self.get_strong_coupling_space(input_states, self.couplings)
-
-    @classmethod
-    def canonicalize(cls, spec):
-        try:
-            return {k:GroupsDegeneracySpec._validate_grp(v) for k,v in spec.items()}
-        except (np.VisibleDeprecationWarning, AttributeError, StopIteration):
-            return None
-
-    @classmethod
-    def get_strong_coupling_space(cls, states: BasisStateSpace, couplings: dict):
-        indexer = SymmetricGroupGenerator(states.ndim)
-        groups = [None] * len(states.indices)
-        for n, i in enumerate(states.indices):
-            if i in couplings:
-                groups[n] = [i] + list(couplings[i].indices)
-            else:
-                groups[n] = [i]
-
-        groups = PermutationRelationGraph.merge_groups([(g, indexer.from_indices(g)) for g in groups])
-
-        return [g[1] for g in groups]
-
 class CallableDegeneracySpec(DegeneracySpec):
     """
 
     """
 
     format = DegenerateSpaceInputFormat.Callable
-    def __init__(self, callable):
+    def __init__(self, callable, **opts):
+        super().__init__(**opts)
         self.callable = callable
 
-    def get_groups(self, input_states, solver=None):
+    def get_groups(self, input_states, solver=None, **kwargs):
         """
         :param input_states:
         :type input_states:
@@ -494,19 +618,22 @@ class CallableDegeneracySpec(DegeneracySpec):
         :return:
         :rtype:
         """
-        return self.callable(input_states, solver=solver)
+        return self.callable(input_states, solver=solver, **kwargs)
 
 class DegenerateMultiStateSpace(BasisMultiStateSpace):
 
     @staticmethod
     def default_group_filter(group,
+                             states=None,
+                             basis=None,
                              corrections=None,
                              threshold=None,
                              energy_cutoff=None,
                              energies=None,
-                             decoupling_overide=100,
+                             decoupling_overide=-1,
                              maximize_groups=True,
-                             target_modes=None
+                             target_modes=None,
+                             threshold_step_size=.1
                              ):
         """
         Excludes modes that differ in only one position, prioritizing states with fewer numbers of quanta
@@ -532,26 +659,58 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
             exc = group.excitations
 
         if corrections is not None:
-            group_inds = corrections.total_basis.find(group)
-            state_inds = corrections.states.find(group, missing_val=-1)
-            kill_spots = (np.arange(len(group_inds))[state_inds > 0], state_inds[state_inds > 0])
-            corr_mat = [np.abs(x[:, group_inds].asarray().T) for x in corrections.wfn_corrections[1:]]
-            max_corr = None
-            for c in corr_mat:
-                c[kill_spots] = 0
-                if max_corr is None:
-                    max_corr = np.max(c, axis=1)
+            if isinstance(corrections, np.ndarray):
+                state_inds = basis.find(group)
+                group_inds = state_inds
+                found_pos = state_inds < len(states)
+            elif hasattr(corrections, 'total_basis'): # PT corrections
+                states = corrections.states
+                basis = corrections.total_basis
+                group_inds = corrections.total_basis.find(group)
+                state_inds = corrections.states.find(group, dtype=int, missing_val=len(states))
+                found_pos = state_inds < len(states)
+                kill_spots = (np.arange(len(group_inds))[found_pos], state_inds[found_pos])
+                corr_mat = [np.abs(x[:, group_inds].asarray().T) for x in corrections.wfn_corrections[1:]]
+                max_corr = None
+                for c in corr_mat:
+                    c[kill_spots] = 0
+                    if max_corr is None:
+                        max_corr = np.max(c, axis=1)
+                    else:
+                        submax = np.max(c, axis=1)
+                        p = submax > max_corr
+                        max_corr[p] = submax[p]
+                corrections = sum(corr_mat)
+                if threshold is not None:
+                    sorting = np.arange(len(max_corr))
                 else:
-                    submax = np.max(c, axis=1)
+                    sorting = np.argsort(-max_corr - 1 * (found_pos))  # prioritize states inside requested packet for marginal cases
+            elif basis is not None and states is not None:
+                group_inds = basis.find(group)
+                state_inds = states.find(group, dtype=int, missing_val=len(states))
+                found_pos = state_inds < len(states)
+                found_states = state_inds[found_pos]
+                kill_spots = (np.arange(len(group_inds))[found_pos], found_states)
+                corrections = np.abs(corrections[:, group_inds].asarray().T)
+                max_corr = None
+                corrections[kill_spots] = 0
+                # print(group.excitations)
+                # print(states.excitations)
+                # raise Exception(corrections)
+                if max_corr is None:
+                    max_corr = np.max(corrections, axis=1)
+                else:
+                    submax = np.max(corrections, axis=1)
                     p = submax > max_corr
                     max_corr[p] = submax[p]
-            if threshold is not None:
-                sorting = np.arange(len(max_corr))
-            else:
-                sorting = np.argsort(-max_corr - 1*(state_inds > 0)) # prioritize states inside requested packet for marginal cases
+                if threshold is not None:
+                    sorting = np.arange(len(max_corr))
+                else:
+                    sorting = np.argsort(-max_corr - 1*found_pos)  # prioritize states inside requested packet for marginal cases
+
         else:
             sorting = np.argsort(inds)
-            corr_mat = None
+            corrections = None
 
         exc = exc[sorting]
         diffs = exc[:, np.newaxis] - exc[np.newaxis, :]  # difference matrix (s, s, m)
@@ -559,7 +718,7 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
         diff_sums = np.sum(np.abs(diffs), axis=2)  # (s, s)
         tots_sums = np.sum(np.abs(tots), axis=2)  # (s, s)
         elims = []
-        if threshold is not None and corr_mat is not None:
+        if threshold is not None and corrections is not None:
             if energy_cutoff is None:
                 bad_pos = np.where(np.logical_and(
                     diff_sums == 1,
@@ -568,14 +727,19 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
             else:
                 e_vec = energies[group_inds]
                 bad_pos = np.where(np.abs(np.subtract.outer(e_vec, e_vec)) > energy_cutoff)
+                # print(">>>", group.excitations)
+                # print(bad_pos)
             if len(bad_pos) > 0 and len(bad_pos[0]) > 0:
                 N = len(group_inds)
-                base_corr = sum(corr_mat)
-                base_vals = base_corr[:, state_inds[state_inds > 0]]
-                base_mat = np.zeros((N, N), dtype=int)
-                base_mat[:, np.where(state_inds > 0)[0]] = base_vals
+                base_corr = corrections
+                base_vals = base_corr[:, state_inds[found_pos]]
+                base_mat = np.zeros((N, N), dtype=float)
+                base_mat[:, np.where(found_pos)[0]] = base_vals
                 base_mat = (base_mat.T + base_mat) - 2*np.diag(np.diag(base_mat))
-                truly_bad = base_mat[bad_pos] < decoupling_overide
+                if decoupling_overide > 0:
+                    truly_bad = base_mat[bad_pos] < decoupling_overide
+                else:
+                    truly_bad = np.full(len(base_mat[bad_pos]), True)
                 # introduce an override so things that are super strongly coupled can remain coupled
                 bad_pos = tuple(b[truly_bad] for b in bad_pos)
                 base_mat[bad_pos] = 0  # refuse to couple problem states
@@ -593,7 +757,7 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
                 groups, A = get_groups(corr_mat)
 
                 target_threshold = threshold
-                step_size = .1
+                step_size = threshold_step_size
                 while (A[bad_pos] > 0).any():
                     threshold = threshold + step_size
                     corr_mat = (base_mat > threshold).astype(int)
@@ -632,13 +796,22 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
 
         return group
 
+    @classmethod
+    def construct_filer(cls, spec=None, **kwargs):
+        if spec is not None:
+            return spec.get_group_filter(**kwargs)
+        else:
+            return lambda group:cls.default_group_filter(group, **kwargs)
+
     @classmethod
     def from_spec(cls,
                   degenerate_states,
                   solver=None,
                   full_basis=None,
                   format=None,
-                  group_filter=None
+                  group_filter=None,
+                  log_groups=False,
+                  **kwargs
                   ):
         """
         Generates a DegenerateMultiStateSpace object from a number
@@ -657,7 +830,9 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
             if not isinstance(degenerate_states, DegeneracySpec):
                 degenerate_states = DegeneracySpec.from_spec(degenerate_states, format=format)
             with logger.block(tag="getting degeneracies"):
-                deg_states = degenerate_states.get_groups(states, solver=solver)
+                deg_states = degenerate_states.get_groups(states, solver=solver, **kwargs)
+                if hasattr(group_filter, 'items'):
+                    group_filter = cls.construct_filer(**dict(group_filter, spec=degenerate_states))
                 new = GroupsDegeneracySpec.canonicalize(deg_states)
                 if new is None:
                     raise ValueError("{} returned invalid degenerate subspace spec {}".format(
@@ -674,11 +849,17 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
                     for s in degenerate_states
                 ]
 
-                logger.log_print(
-                    "{n} degenerate state sets found",
-                    n=len([x for x in degenerate_states if len(x) > 1]),
-                    # s=[x for x in degenerate_states if len(x) > 1]
-                )
+                if log_groups:
+                    with logger.block(tag="initial degenerate groups:"):
+                        for g in degenerate_states:
+                            if len(g) > 1:
+                                logger.log_print(str(g.excitations).splitlines())
+                else:
+                    logger.log_print(
+                        "{n} degenerate state sets found",
+                        n=len([x for x in degenerate_states if len(x) > 1]),
+                        # s=[x for x in degenerate_states if len(x) > 1]
+                    )
 
         # build groups of degenerate states for use later
         if degenerate_states is None:
@@ -701,6 +882,8 @@ class DegenerateMultiStateSpace(BasisMultiStateSpace):
         # now turn these into proper BasisStateSpace objects so we can work with them more easily
         ugh = []
         for i,g in enumerate(groups):
+            if g is None:
+                continue #???
             # g = np.sort(np.array(g))
             if not isinstance(g, BasisStateSpace):
                 # if np.any(np.asanyarray(g) < 0):
